@@ -24,9 +24,9 @@ import android.content.SharedPreferences;
 import android.os.Environment;
 import android.util.*;
 
-import org.altusmetrum.altoslib_6.*;
+import org.altusmetrum.altoslib_11.*;
 
-public class AltosDroidPreferencesBackend implements AltosPreferencesBackend {
+public class AltosDroidPreferencesBackend extends AltosPreferencesBackend {
 	public final static String        NAME    = "org.altusmetrum.AltosDroid";
 	private Context                   context = null;
 	private SharedPreferences         prefs   = null;
@@ -44,10 +44,17 @@ public class AltosDroidPreferencesBackend implements AltosPreferencesBackend {
 
 	public String[] keys() {
 		Map<String, ?> all = prefs.getAll();
-		return (String[])all.keySet().toArray();
+		Object[] ao = all.keySet().toArray();
+
+		String[] as = new String[ao.length];
+		for (int i = 0; i < ao.length; i++)
+			as[i] = (String) ao[i];
+		return as;
 	}
 
 	public AltosPreferencesBackend node(String key) {
+		if (!nodeExists(key))
+			putBoolean(key, true);
 		return new AltosDroidPreferencesBackend(context, key);
 	}
 
@@ -104,6 +111,7 @@ public class AltosDroidPreferencesBackend implements AltosPreferencesBackend {
 	}
 
 	public void remove(String key) {
+		AltosDebug.debug("remove preference %s\n", key);
 		editor.remove(key);
 	}
 
@@ -114,4 +122,8 @@ public class AltosDroidPreferencesBackend implements AltosPreferencesBackend {
 	public File homeDirectory() {
 		return Environment.getExternalStorageDirectory();
 	}
+
+	public void debug(String format, Object ... arguments) {
+		AltosDebug.debug(format, arguments);
+	}
 }
