@@ -17,7 +17,7 @@
 
 package org.altusmetrum.AltosDroid;
 
-import org.altusmetrum.altoslib_6.*;
+import org.altusmetrum.altoslib_11.*;
 import android.location.Location;
 import android.app.Activity;
 import android.graphics.Color;
@@ -26,21 +26,22 @@ import android.support.v4.app.Fragment;
 import android.support.v4.app.FragmentTransaction;
 import android.support.v4.app.FragmentManager;
 import android.location.Location;
-import android.util.Log;
 import android.widget.TextView;
 
 public abstract class AltosDroidTab extends Fragment implements AltosUnitsListener {
+	TelemetryState		last_telem_state;
 	AltosState		last_state;
 	AltosGreatCircle	last_from_receiver;
 	Location		last_receiver;
+	AltosDroid		altos_droid;
 
-	public abstract void show(AltosState state, AltosGreatCircle from_receiver, Location receiver);
+	public abstract void show(TelemetryState telem_state, AltosState state, AltosGreatCircle from_receiver, Location receiver);
 
 	public abstract String tab_name();
 
 	public void units_changed(boolean imperial_units) {
-		if (!isHidden() && last_state != null)
-			show(last_state, last_from_receiver, last_receiver);
+		if (!isHidden())
+			show(last_telem_state, last_state, last_from_receiver, last_receiver);
 	}
 
 	public void set_value(TextView text_view,
@@ -55,29 +56,46 @@ public abstract class AltosDroidTab extends Fragment implements AltosUnitsListen
 
 	public void set_visible(boolean visible) {
 		FragmentTransaction	ft = AltosDroid.fm.beginTransaction();
+		AltosDebug.debug("set visible %b %s\n", visible, tab_name());
 		if (visible) {
-			AltosState		state = last_state;
-			AltosGreatCircle	from_receiver = last_from_receiver;
-			Location		receiver = last_receiver;
-
-			show(state, from_receiver, receiver);
 			ft.show(this);
+			show(last_telem_state, last_state, last_from_receiver, last_receiver);
 		} else
 			ft.hide(this);
-		ft.commit();
+		ft.commitAllowingStateLoss();
 	}
 
-	public void update_ui(AltosState state, AltosGreatCircle from_receiver, Location receiver, boolean is_current) {
+	@Override
+	public void onAttach(Activity activity) {
+		super.onAttach(activity);
+		altos_droid = (AltosDroid) activity;
+		altos_droid.registerTab(this);
+	}
+
+	@Override
+	public void onDetach() {
+		super.onDetach();
+		altos_droid.unregisterTab(this);
+		altos_droid = null;
+	}
+
+	@Override
+	public void onResume() {
+		super.onResume();
+		AltosDebug.debug("onResume tab %s\n", tab_name());
+		set_visible(true);
+	}
+
+	public void update_ui(TelemetryState telem_state, AltosState state,
+			      AltosGreatCircle from_receiver, Location receiver, boolean is_current)
+	{
+		last_telem_state = telem_state;
 		last_state = state;
 		last_from_receiver = from_receiver;
 		last_receiver = receiver;
-		if (is_current) {
-			if (AltosDroid.D) Log.d(AltosDroid.TAG, String.format("%s: visible, performing update", tab_name()));
-
-			show(state, from_receiver, receiver);
-		} else {
-			if (AltosDroid.D) Log.d(AltosDroid.TAG, String.format("%s: not visible, skipping update", tab_name()));
+		if (is_current)
+			show(telem_state, state, from_receiver, receiver);
+		else
 			return;
-		}
 	}
 }
