@@ -15,11 +15,11 @@
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
  */
 
-package org.altusmetrum.altoslib_6;
+package org.altusmetrum.altoslib_11;
 
 import java.io.*;
 
-public class AltosCompanion implements Serializable {
+public class AltosCompanion {
 	public final static int	board_id_telescience = 0x0a;
 	public final static int	MAX_CHANNELS = 12;
 
@@ -37,4 +37,9 @@ public class AltosCompanion implements Serializable {
 			channels = MAX_CHANNELS;
 		companion_data = new int[channels];
 	}
+
+	public AltosCompanion() {
+		channels = 0;
+		companion_data = new int[0];
+	}
 }
