@@ -15,7 +15,7 @@
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
  */
 
-package org.altusmetrum.altoslib_6;
+package org.altusmetrum.altoslib_11;
 
 import java.util.*;
 import java.text.*;
@@ -75,6 +75,7 @@ public class AltosConfigData implements Iterable<String> {
 	/* HAS_APRS */
 	public int		aprs_interval;
 	public int		aprs_ssid;
+	public int		aprs_format;
 
 	/* HAS_BEEP */
 	public int		beep;
@@ -203,7 +204,7 @@ public class AltosConfigData implements Iterable<String> {
 
 		for (int i = 0; i < parts.length; i++) {
 			try {
-				r[i] = AltosLib.fromdec(parts[i]);
+				r[i] = (int) AltosLib.fromdec(parts[i]);
 			} catch (NumberFormatException n) {
 				r[i] = 0;
 			}
@@ -270,6 +271,7 @@ public class AltosConfigData implements Iterable<String> {
 
 		aprs_interval = -1;
 		aprs_ssid = -1;
+		aprs_format = -1;
 
 		beep = -1;
 
@@ -370,6 +372,7 @@ public class AltosConfigData implements Iterable<String> {
 		/* HAS_APRS */
 		try { aprs_interval = get_int(line, "APRS interval:"); } catch (Exception e) {}
 		try { aprs_ssid = get_int(line, "APRS SSID:"); } catch (Exception e) {}
+		try { aprs_format = get_int(line, "APRS format:"); } catch (Exception e) {}
 
 		/* HAS_BEEP */
 		try { beep = get_int(line, "Beeper setting:"); } catch (Exception e) {}
@@ -473,6 +476,18 @@ public class AltosConfigData implements Iterable<String> {
 	}
 
 
+	public boolean mma655x_inverted() throws AltosUnknownProduct {
+		if (product.startsWith("EasyMega-v1"))
+			return false;
+		if (product.startsWith("TeleMetrum-v2"))
+			return true;
+		if (product.startsWith("TeleMega-v2"))
+			return false;
+		if (product.startsWith("TeleMega-v1"))
+			return false;
+		throw new AltosUnknownProduct(product);
+	}
+
 	public void get_values(AltosConfigValues source) throws AltosConfigDataException {
 
 		/* HAS_FLIGHT */
@@ -490,8 +505,6 @@ public class AltosConfigData implements Iterable<String> {
 			radio_enable = source.radio_enable();
 		if (callsign != null)
 			callsign = source.callsign();
-		if (radio_calibration >= 0)
-			radio_calibration = source.radio_calibration();
 		if (telemetry_rate >= 0)
 			telemetry_rate = source.telemetry_rate();
 
@@ -518,6 +531,8 @@ public class AltosConfigData implements Iterable<String> {
 			aprs_interval = source.aprs_interval();
 		if (aprs_ssid >= 0)
 			aprs_ssid = source.aprs_ssid();
+		if (aprs_format >= 0)
+			aprs_format = source.aprs_format();
 
 		/* HAS_BEEP */
 		if (beep >= 0)
@@ -572,6 +587,7 @@ public class AltosConfigData implements Iterable<String> {
 		dest.set_pyro_firing_time(pyro_firing_time);
 		dest.set_aprs_interval(aprs_interval);
 		dest.set_aprs_ssid(aprs_ssid);
+		dest.set_aprs_format(aprs_format);
 		dest.set_beep(beep);
 		dest.set_tracker_motion(tracker_motion);
 		dest.set_tracker_interval(tracker_interval);
@@ -595,10 +611,6 @@ public class AltosConfigData implements Iterable<String> {
 		if (apogee_lockout >= 0)
 			link.printf("c L %d\n", apogee_lockout);
 
-		/* Don't mess with radio calibration when remote */
-		if (radio_calibration > 0 && !remote)
-			link.printf("c f %d\n", radio_calibration);
-
 		/* HAS_RADIO */
 		if (has_frequency()) {
 			boolean has_frequency = radio_frequency >= 0;
@@ -674,6 +686,8 @@ public class AltosConfigData implements Iterable<String> {
 			link.printf("c A %d\n", aprs_interval);
 		if (aprs_ssid >= 0)
 			link.printf("c S %d\n", aprs_ssid);
+		if (aprs_format >= 0)
+			link.printf("c C %d\n", aprs_format);
 
 		/* HAS_BEEP */
 		if (beep >= 0)
