@@ -18,7 +18,7 @@
 /*
  * Sensor data conversion functions
  */
-package org.altusmetrum.altoslib_6;
+package org.altusmetrum.altoslib_11;
 
 public class AltosConvert {
 	/*
@@ -230,6 +230,12 @@ public class AltosConvert {
 		return sensor / 32767.0 * supply * (5.6 + 10.0) / 10.0;
 	}
 
+	static double tele_bt_3_battery(int raw) {
+		if (raw == AltosLib.MISSING)
+			return AltosLib.MISSING;
+		return 3.3 * mega_adc(raw) * (5.1 + 10.0) / 10.0;
+	}
+
 	static double easy_mini_voltage(int sensor, int serial) {
 		double	supply = 3.3;
 		double	diode_offset = 0.0;
@@ -319,6 +325,14 @@ public class AltosConvert {
 		return miles_to_meters(mps) / 3600;
  	}
 
+	public static double mps_to_fps(double mps) {
+		return meters_to_miles(mps) * 5280;
+ 	}
+
+	public static double fps_to_mps(double mps) {
+		return miles_to_meters(mps) / 5280;
+ 	}
+
 	public static double meters_to_mach(double meters) {
 		return meters / 343;		/* something close to mach at usual rocket sites */
 	}
