@@ -15,7 +15,7 @@
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
  */
 
-package org.altusmetrum.altoslib_6;
+package org.altusmetrum.altoslib_11;
 
 import java.io.*;
 import java.util.*;
@@ -72,7 +72,7 @@ public class AltosEepromFile extends AltosStateIterable {
 		headers = new AltosEepromIterable(AltosEepromHeader.read(input));
 
 		start = headers.state();
-		if (start.state != AltosLib.ao_flight_stateless)
+		if (start.state() != AltosLib.ao_flight_stateless)
 			start.set_state(AltosLib.ao_flight_pad);
 
 		if (start.log_format == AltosLib.MISSING) {
@@ -89,7 +89,7 @@ public class AltosEepromFile extends AltosStateIterable {
 			body = new AltosEepromIterable(AltosEepromTM.read(input));
 			break;
 		case AltosLib.AO_LOG_FORMAT_TINY:
-			body = new AltosEepromIterable(AltosEepromTm.read(input));
+			body = new AltosEepromIterable(AltosEepromTMini.read(input));
 			break;
 		case AltosLib.AO_LOG_FORMAT_TELEMETRY:
 		case AltosLib.AO_LOG_FORMAT_TELESCIENCE:
@@ -117,7 +117,7 @@ public class AltosEepromFile extends AltosStateIterable {
 		for (AltosEeprom eeprom : body) {
 			eeprom.update_state(state);
 			state.finish_update();
-			if (state.state >= AltosLib.ao_flight_boost) {
+			if (state.state() >= AltosLib.ao_flight_boost) {
 				start.set_boost_tick(state.tick);
 				break;
 			}
