@@ -15,12 +15,29 @@
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
  */
 
-package org.altusmetrum.altoslib_6;
+package org.altusmetrum.altoslib_11;
+
+import java.io.*;
+import java.util.*;
+import java.text.*;
 
 public class AltosFrequency {
 	public double	frequency;
 	public String	description;
 
+	public int hashCode() {
+		return new Double(frequency).hashCode();
+	}
+
+	public boolean equals(Object o) {
+		if (o == null)
+			return false;
+		if (!(o instanceof AltosFrequency))
+			return false;
+		AltosFrequency other = (AltosFrequency) o;
+		return other.frequency == frequency;
+	}
+
 	public String toString() {
 		return String.format("%7.3f MHz %-20s",
 				     frequency, description);
@@ -31,12 +48,15 @@ public class AltosFrequency {
 				     frequency, description);
 	}
 
+	public String frequency_string() {
+		return String.format("%7.3f", frequency);
+	}
+
 	public boolean close(double f) {
 		double	diff = Math.abs(frequency - f);
 
 		return diff < 0.010;
 	}
-
 	public AltosFrequency(double f, String d) {
 		frequency = f;
 		description = d;
