@@ -15,7 +15,7 @@
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
  */
 
-package org.altusmetrum.altoslib_6;
+package org.altusmetrum.altoslib_11;
 
 import java.util.*;
 import java.io.*;
@@ -24,6 +24,7 @@ import java.util.concurrent.*;
 public class AltosIgnite {
 	AltosLink	link;
 	boolean		remote;
+	boolean		close_on_exit;
 	boolean		link_started;
 	boolean		have_npyro = false;
 	int		npyro;
@@ -180,14 +181,18 @@ public class AltosIgnite {
 
 	public void close() throws InterruptedException {
 		stop_link();
-		link.close();
+		if (close_on_exit)
+			link.close();
 		link = null;
 	}
 
-	public AltosIgnite(AltosLink in_link, boolean in_remote)
-		throws FileNotFoundException, TimeoutException, InterruptedException {
-
+	public AltosIgnite(AltosLink in_link, boolean in_remote, boolean in_close_on_exit) {
 		link = in_link;
 		remote = in_remote;
+		close_on_exit = in_close_on_exit;
 	}
-}
\ No newline at end of file
+
+	public AltosIgnite(AltosLink in_link, boolean in_remote) {
+		this(in_link, in_remote, true);
+	}
+}
