@@ -15,7 +15,7 @@
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
  */
 
-package org.altusmetrum.altoslib_6;
+package org.altusmetrum.altoslib_11;
 
 import java.util.*;
 import java.io.*;
@@ -190,7 +190,15 @@ public class AltosLib {
 		38400, 9600, 2400
 	};
 
+	public static final int ao_aprs_format_compressed = 0;
+	public static final int ao_aprs_format_uncompressed = 1;
+
+	public static final String[] ao_aprs_format_name = {
+		"Compressed", "Uncompressed"
+	};
+
 	public static final String launch_sites_url = "http://www.altusmetrum.org/AltOS/launch-sites.txt";
+	public static final String launch_sites_env = "LAUNCH_SITES";
 //	public static final String launch_sites_url = "file:///home/keithp/misc/text/altusmetrum/AltOS/launch-sites.txt";
 
 	public static final int ao_telemetry_standard_len = 32;
@@ -485,9 +493,10 @@ public class AltosLib {
 		return r;
 	}
 
-	public static int fromdec(String s) throws NumberFormatException {
-		int c, v = 0;
-		int sign = 1;
+	public static long fromdec(String s) throws NumberFormatException {
+		int c;
+		long v = 0;
+		long sign = 1;
 		for (int i = 0; i < s.length(); i++) {
 			c = s.charAt(i);
 			if (i == 0 && c == '-') {
