@@ -15,11 +15,11 @@
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
  */
 
-package org.altusmetrum.altoslib_6;
+package org.altusmetrum.altoslib_11;
 
 import java.io.*;
 
-public class AltosListenerState implements Serializable {
+public class AltosListenerState {
 	public int	crc_errors;
 	public double	battery;
 	public boolean	running;
