@@ -15,8 +15,9 @@
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
  */
 
-package org.altusmetrum.altoslib_6;
+package org.altusmetrum.altoslib_11;
 
+import java.util.*;
 import java.text.*;
 
 public class AltosParse {
@@ -25,6 +26,14 @@ public class AltosParse {
 	}
 
 	public static int parse_int(String v) throws ParseException {
+		try {
+			return (int) AltosLib.fromdec(v);
+		} catch (NumberFormatException e) {
+			throw new ParseException("error parsing int " + v, 0);
+		}
+	}
+
+	public static long parse_long(String v) throws ParseException {
 		try {
 			return AltosLib.fromdec(v);
 		} catch (NumberFormatException e) {
@@ -40,14 +49,52 @@ public class AltosParse {
 		}
 	}
 
-	public static double parse_double(String v) throws ParseException {
+	static NumberFormat get_nf_locale() {
+		NumberFormat nf = NumberFormat.getInstance();
+		nf.setParseIntegerOnly(false);
+		nf.setGroupingUsed(false);
+		return nf;
+	}
+
+	static NumberFormat nf_locale = get_nf_locale();
+
+	static NumberFormat get_nf_net() {
+		NumberFormat nf = NumberFormat.getInstance(Locale.ROOT);
+		nf.setParseIntegerOnly(false);
+		nf.setGroupingUsed(false);
+		return nf;
+	}
+
+	static NumberFormat nf_net = get_nf_net();
+
+	public static double parse_double_locale(String str) throws ParseException {
 		try {
-			return Double.parseDouble(v);
-		} catch (NumberFormatException e) {
-			throw new ParseException("error parsing double " + v, 0);
+			return nf_locale.parse(str.trim()).doubleValue();
+		} catch (ParseException pe) {
+			throw new ParseException("error parsing double " + str, 0);
 		}
 	}
 
+	public static String format_double_locale(double number) {
+		return nf_locale.format(number);
+	}
+
+	public static double parse_double_net(String str) throws ParseException {
+		try {
+			String t = str.trim();
+//			System.out.printf("Parse string \"%s\" trim \"%s\"\n", str, t);
+			return nf_net.parse(t).doubleValue();
+		} catch (ParseException pe) {
+			throw new ParseException("error parsing double " + str, 0);
+		}
+	}
+
+	public static String format_double_net(double number) {
+		String ret = nf_net.format(number);
+//		System.out.printf("format double %f \"%s\"\n", number, ret);
+		return ret;
+	}
+
 	public static double parse_coord(String coord) throws ParseException {
 		String[]	dsf = coord.split("\\D+");
 
