@@ -15,10 +15,11 @@
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
  */
 
-package org.altusmetrum.altoslib_6;
+package org.altusmetrum.altoslib_11;
 
 import java.io.*;
 import java.util.*;
+import java.text.*;
 
 public class AltosPreferences {
 	public static AltosPreferencesBackend backend = null;
@@ -42,7 +43,9 @@ public class AltosPreferences {
 	public final static String logfilePreferenceFormat = "LOGFILE-%d";
 
 	/* state preference name */
+	public final static String statePreferenceHead = "STATE-";
 	public final static String statePreferenceFormat = "STATE-%d";
+	public final static String statePreferenceLatest = "STATE-LATEST";
 
 	/* voice preference name */
 	public final static String voicePreference = "VOICE";
@@ -113,30 +116,52 @@ public class AltosPreferences {
 	public final static String	frequency_count = "COUNT";
 	public final static String	frequency_format = "FREQUENCY-%d";
 	public final static String	description_format = "DESCRIPTION-%d";
+	public final static String	frequenciesPreference = "FREQUENCIES-1";
 
 	/* Units preference */
 
 	public final static String	unitsPreference = "IMPERIAL-UNITS";
 
+	/* Maps cache size preference name */
+	final static String mapCachePreference = "MAP-CACHE";
+
+	static LinkedList<AltosMapCacheListener> map_cache_listeners;
+
+	public static int map_cache = 9;
+
+	final static String mapTypePreference = "MAP-TYPE";
+	static int	map_type;
+
 	public static AltosFrequency[] load_common_frequencies() {
 		AltosFrequency[] frequencies = null;
-		boolean	existing = false;
-		existing = backend.nodeExists(common_frequencies_node_name);
 
-		if (existing) {
-			AltosPreferencesBackend	node = backend.node(common_frequencies_node_name);
-			int		count = node.getInt(frequency_count, 0);
+		try {
+			AltosJson json = AltosJson.fromString(backend.getString(frequenciesPreference,
+										null));
+			frequencies = (AltosFrequency[]) json.make(frequencies.getClass());
+		} catch (Exception e) {
+		}
 
-			frequencies = new AltosFrequency[count];
-			for (int i = 0; i < count; i++) {
-				double	frequency;
-				String	description;
+		if (frequencies == null) {
+			if (backend.nodeExists(common_frequencies_node_name)) {
+				AltosPreferencesBackend	node = backend.node(common_frequencies_node_name);
+				int		count = node.getInt(frequency_count, 0);
 
-				frequency = node.getDouble(String.format(frequency_format, i), 0.0);
-				description = node.getString(String.format(description_format, i), null);
-				frequencies[i] = new AltosFrequency(frequency, description);
+				if (count > 0) {
+					frequencies = new AltosFrequency[count];
+					for (int i = 0; i < count; i++) {
+						double	frequency;
+						String	description;
+
+						frequency = node.getDouble(String.format(frequency_format, i), 0.0);
+						description = node.getString(String.format(description_format, i), null);
+						frequencies[i] = new AltosFrequency(frequency, description);
+					}
+				}
 			}
-		} else {
+		}
+
+		if (frequencies == null) {
 			frequencies = new AltosFrequency[10];
 			for (int i = 0; i < 10; i++) {
 				frequencies[i] = new AltosFrequency(434.550 + i * .1,
@@ -146,15 +171,12 @@ public class AltosPreferences {
 		return frequencies;
 	}
 
-	public static void save_common_frequencies(AltosFrequency[] frequencies) {
-		AltosPreferencesBackend	node = backend.node(common_frequencies_node_name);
-
-		node.putInt(frequency_count, frequencies.length);
-		for (int i = 0; i < frequencies.length; i++) {
-			node.putDouble(String.format(frequency_format, i), frequencies[i].frequency);
-			node.putString(String.format(description_format, i), frequencies[i].description);
-		}
+	public static void save_common_frequencies() {
+		AltosJson	json = new AltosJson(common_frequencies);
+		backend.putString(frequenciesPreference, json.toString());
+		flush_preferences();
 	}
+
 	public static int launcher_serial;
 
 	public static int launcher_channel;
@@ -208,6 +230,10 @@ public class AltosPreferences {
 		common_frequencies = load_common_frequencies();
 
 		AltosConvert.imperial_units = backend.getBoolean(unitsPreference, false);
+
+		map_cache = backend.getInt(mapCachePreference, 9);
+		map_cache_listeners = new LinkedList<AltosMapCacheListener>();
+		map_type = backend.getInt(mapTypePreference, AltosMap.maptype_hybrid);
 	}
 
 	public static void flush_preferences() {
@@ -336,46 +362,56 @@ public class AltosPreferences {
 		}
 	}
 
-	public static void set_state(int serial, AltosState state, AltosListenerState listener_state) {
+	public static void set_state(AltosState state) {
 
-		ByteArrayOutputStream baos = new ByteArrayOutputStream();
-
-		try {
-			ObjectOutputStream oos = new ObjectOutputStream(baos);
-
-			AltosSavedState	saved_state = new AltosSavedState(state, listener_state);
-			oos.writeObject(saved_state);
-
-			byte[] bytes = baos.toByteArray();
-
-			synchronized(backend) {
-				backend.putBytes(String.format(statePreferenceFormat, serial), bytes);
-				flush_preferences();
-			}
-		} catch (IOException ie) {
+		synchronized(backend) {
+			backend.putJson(String.format(statePreferenceFormat, state.serial), new AltosJson(state));
+			backend.putInt(statePreferenceLatest, state.serial);
+			flush_preferences();
 		}
 	}
 
-	public static AltosSavedState state(int serial) {
-		byte[] bytes = null;
+	public static ArrayList<Integer> list_states() {
+		String[]		keys = backend.keys();
+		ArrayList<Integer>	states = new ArrayList<Integer>();
 
+		for (String key : keys) {
+			if (key.startsWith(statePreferenceHead)) {
+				try {
+					int serial = AltosParse.parse_int(key.substring(statePreferenceHead.length()));
+					states.add(serial);
+				} catch (ParseException pe) {
+				}
+			}
+		}
+		return states;
+	}
+
+	public static void remove_state(int serial) {
 		synchronized(backend) {
-			bytes = backend.getBytes(String.format(statePreferenceFormat, serial), null);
+			backend.remove(String.format(statePreferenceFormat, serial));
+			flush_preferences();
 		}
+	}
 
-		if (bytes == null)
+	public static int latest_state() {
+		int	latest = 0;
+		synchronized (backend) {
+			latest = backend.getInt(statePreferenceLatest, 0);
+		}
+		return latest;
+	}
+
+	public static AltosState state(int serial) {
+		synchronized(backend) {
+			try {
+				AltosJson json = backend.getJson(String.format(statePreferenceFormat, serial));
+				if (json != null)
+					return (AltosState) (json.make(AltosState.class));
+			} catch (Exception e) {
+			}
 			return null;
-
-		ByteArrayInputStream bais = new ByteArrayInputStream(bytes);
-
-		try {
-			ObjectInputStream ois = new ObjectInputStream(bais);
-			AltosSavedState saved_state = (AltosSavedState) ois.readObject();
-			return saved_state;
-		} catch (IOException ie) {
-		} catch (ClassNotFoundException ce) {
 		}
-		return null;
 	}
 
 	public static void set_scanning_telemetry(int new_scanning_telemetry) {
@@ -491,8 +527,8 @@ public class AltosPreferences {
 	public static void set_common_frequencies(AltosFrequency[] frequencies) {
 		synchronized(backend) {
 			common_frequencies = frequencies;
-			save_common_frequencies(frequencies);
-			flush_preferences();
+
+			save_common_frequencies();
 		}
 	}
 
@@ -548,4 +584,68 @@ public class AltosPreferences {
 			units_listeners.remove(l);
 		}
 	}
+
+
+	public static void register_map_cache_listener(AltosMapCacheListener l) {
+		synchronized(backend) {
+			map_cache_listeners.add(l);
+		}
+	}
+
+	public static void unregister_map_cache_listener(AltosMapCacheListener l) {
+		synchronized (backend) {
+			map_cache_listeners.remove(l);
+		}
+	}
+
+	public static void set_map_cache(int new_map_cache) {
+		synchronized(backend) {
+			map_cache = new_map_cache;
+			backend.putInt(mapCachePreference, map_cache);
+			flush_preferences();
+			for (AltosMapCacheListener l: map_cache_listeners)
+				l.map_cache_changed(map_cache);
+		}
+	}
+
+	public static int map_cache() {
+		synchronized(backend) {
+			return map_cache;
+		}
+	}
+
+	static LinkedList<AltosMapTypeListener> map_type_listeners;
+
+	public static void set_map_type(int map_type) {
+		synchronized(backend) {
+			AltosPreferences.map_type = map_type;
+			backend.putInt(mapTypePreference, map_type);
+			flush_preferences();
+		}
+		if (map_type_listeners != null) {
+			for (AltosMapTypeListener l : map_type_listeners) {
+				l.map_type_changed(map_type);
+			}
+		}
+	}
+
+	public static int map_type() {
+		synchronized(backend) {
+			return map_type;
+		}
+	}
+
+	public static void register_map_type_listener(AltosMapTypeListener l) {
+		synchronized(backend) {
+			if (map_type_listeners == null)
+				map_type_listeners = new LinkedList<AltosMapTypeListener>();
+			map_type_listeners.add(l);
+		}
+	}
+
+	public static void unregister_map_type_listener(AltosMapTypeListener l) {
+		synchronized(backend) {
+			map_type_listeners.remove(l);
+		}
+	}
 }
