@@ -15,7 +15,7 @@
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
  */
 
-package org.altusmetrum.altoslib_6;
+package org.altusmetrum.altoslib_11;
 
 import java.io.*;
 import java.util.*;
@@ -39,8 +39,8 @@ public class AltosReplayReader extends AltosFlightReader {
 
 	public void update(AltosState state) throws InterruptedException {
 		/* Make it run in realtime after the rocket leaves the pad */
-		if (state.state > AltosLib.ao_flight_pad && state.time_change > 0)
-			Thread.sleep((int) (Math.min(state.time_change,10) * 100));
+		if (state.state() > AltosLib.ao_flight_pad && state.time_change > 0)
+			Thread.sleep((int) (Math.min(state.time_change,10) * 1000));
 		state.set_received_time(System.currentTimeMillis());
 	}
 
