@@ -15,35 +15,35 @@
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
  */
 
-package org.altusmetrum.altoslib_6;
+package org.altusmetrum.altoslib_11;
 
 public class AltosSpeed extends AltosUnits {
 
 	public double value(double v, boolean imperial_units) {
 		if (imperial_units)
-			return AltosConvert.meters_to_mph(v);
+			return AltosConvert.mps_to_fps(v);
 		return v;
 	}
 
 	public double inverse(double v, boolean imperial_units) {
 		if (imperial_units)
-			return AltosConvert.mph_to_meters(v);
+			return AltosConvert.fps_to_mps(v);
 		return v;
 	}
 
 	public String show_units(boolean imperial_units) {
 		if (imperial_units)
-			return "mph";
+			return "fps";
 		return "m/s";
 	}
 
 	public String say_units(boolean imperial_units) {
 		if (imperial_units)
-			return "miles per hour";
+			return "feet per second";
 		return "meters per second";
 	}
 
 	public int show_fraction(int width, boolean imperial_units) {
 		return width / 9;
 	}
-}
\ No newline at end of file
+}
