@@ -15,8 +15,8 @@
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
  */
 
-package org.altusmetrum.altoslib_6;
+package org.altusmetrum.altoslib_11;
 
 public interface AltosStateUpdate {
-	public void	update_state(AltosState state) throws InterruptedException;
-}
\ No newline at end of file
+	public void	update_state(AltosState state) throws InterruptedException, AltosUnknownProduct;
+}
