@@ -19,8 +19,8 @@ package altosui;
 
 import java.awt.*;
 import javax.swing.*;
-import org.altusmetrum.altoslib_6.*;
-import org.altusmetrum.altosuilib_6.*;
+import org.altusmetrum.altoslib_11.*;
+import org.altusmetrum.altosuilib_11.*;
 
 public class AltosFlightStatus extends JComponent implements AltosFlightDisplay {
 	GridBagLayout	layout;
@@ -173,14 +173,14 @@ public class AltosFlightStatus extends JComponent implements AltosFlightDisplay
 		int	last_state = -1;
 
 		void show(AltosState state, AltosListenerState listener_state) {
-			if (state.state != last_state) {
-				if (state.state == AltosLib.ao_flight_stateless)
+			if (state.state() != last_state) {
+				if (state.state() == AltosLib.ao_flight_stateless)
 					hide();
 				else {
 					show();
 					value.setText(state.state_name());
 				}
-				last_state = state.state;
+				last_state = state.state();
 			}
 		}
 
