@@ -18,8 +18,8 @@
 package altosui;
 
 import java.util.*;
-import org.altusmetrum.altoslib_6.*;
-import org.altusmetrum.altosuilib_6.*;
+import org.altusmetrum.altoslib_11.*;
+import org.altusmetrum.altosuilib_11.*;
 
 public class AltosPad extends AltosUIFlightTab {
 
@@ -49,10 +49,10 @@ public class AltosPad extends AltosUIFlightTab {
 				hide();
 			} else {
 				if (state.flight != 0) {
-					if (state.state <= Altos.ao_flight_pad)
+					if (state.state() <= Altos.ao_flight_pad)
 						show("Ready to record");
-					else if (state.state < Altos.ao_flight_landed ||
-						 state.state == AltosLib.ao_flight_stateless)
+					else if (state.state() < Altos.ao_flight_landed ||
+						 state.state() == AltosLib.ao_flight_stateless)
 						show("Recording data");
 					else
 						show("Recorded data");
@@ -101,15 +101,24 @@ public class AltosPad extends AltosUIFlightTab {
 
 	class ReceiverBattery extends AltosUIVoltageIndicator {
 
-		public double voltage(AltosState state) { return AltosLib.MISSING; }
+		double	last_voltage = AltosLib.MISSING;
+
+		public double voltage(AltosState state) {
+			return last_voltage;
+		}
 
-		public boolean hide(double v) { return v == AltosLib.MISSING; }
 		public double good() { return AltosLib.ao_battery_good; }
 
+		public boolean hide(AltosState state, AltosListenerState listener_state, int i) {
+			return value(state, listener_state, i) == AltosLib.MISSING;
+		}
+
 		public double value(AltosState state, AltosListenerState listener_state, int i) {
 			if (listener_state == null)
-				return AltosLib.MISSING;
-			return listener_state.battery;
+				last_voltage = AltosLib.MISSING;
+			else
+				last_voltage = listener_state.battery;
+			return last_voltage;
 		}
 
 		public ReceiverBattery (AltosUIFlightTab container, int y) {
@@ -118,8 +127,8 @@ public class AltosPad extends AltosUIFlightTab {
 	}
 
 	boolean report_pad(AltosState state) {
-		if ((state.state == AltosLib.ao_flight_stateless ||
-		     state.state < AltosLib.ao_flight_pad) &&
+		if ((state.state() == AltosLib.ao_flight_stateless ||
+		     state.state() < AltosLib.ao_flight_pad) &&
 		    state.gps != null &&
 		    state.gps.lat != AltosLib.MISSING)
 		{
@@ -202,40 +211,26 @@ public class AltosPad extends AltosUIFlightTab {
 		}
 	}
 
-	class PadAlt extends AltosUIIndicator {
+	class PadAlt extends AltosUIUnitsIndicator {
 
-		double	last_alt = AltosLib.MISSING - 1;
-
-		public void show (AltosState state, AltosListenerState listener_state) {
-			double alt = AltosLib.MISSING;
-			String label = null;
-
-			if (state != null) {
-				if (report_pad(state)) {
-					alt = state.pad_alt;
-					label = "Pad Altitude";
-				} else {
-					alt = state.gps.alt;
-					label = "Altitude";
-				}
-			}
-			if (alt != last_alt) {
-				if (alt != AltosLib.MISSING) {
-					show(AltosConvert.height.show(5, alt));
-					set_label(label);
-				} else
-					hide();
-				last_alt = alt;
-			}
+		public double value(AltosState state, int i) {
+			if (report_pad(state))
+				return state.pad_alt;
+			else
+				return state.gps.alt;
 		}
 
-		public void reset() {
-			super.reset();
-			last_alt =  AltosLib.MISSING - 1;
+		public void show (AltosState state, AltosListenerState listener_state) {
+			String label = "Altitude";
+
+			if (state != null && report_pad(state))
+				label = "Pad Altitude";
+			set_label(label);
+			super.show(state, listener_state);
 		}
 
 		public PadAlt (AltosUIFlightTab container, int y) {
-			super (container, y, "Pad Altitude", 1, false, 2);
+			super (container, y, AltosConvert.height, "Pad Altitude", 1, false, 2);
 		}
 	}
 	public String getName() { return "Pad"; }
@@ -243,12 +238,12 @@ public class AltosPad extends AltosUIFlightTab {
 	public AltosPad() {
 		int y = 0;
 		add(new Battery(this, y++));
+		add(new ReceiverBattery(this, y++));
 		add(new Apogee(this, y++));
 		add(new Main(this, y++));
 		add(new LoggingReady(this, y++));
 		add(new GPSLocked(this, y++));
 		add(new GPSReady(this, y++));
-		add(new ReceiverBattery(this, y++));
 		add(new PadLat(this, y++));
 		add(new PadLon(this, y++));
 		add(new PadAlt(this, y++));
