@@ -15,7 +15,7 @@
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
  */
 
-package org.altusmetrum.altosuilib_6;
+package org.altusmetrum.altosuilib_11;
 
 import java.awt.*;
 import java.awt.event.*;
@@ -23,8 +23,13 @@ import javax.swing.*;
 import java.util.*;
 
 class AltosUIFrameListener extends WindowAdapter {
+	@Override
 	public void windowClosing (WindowEvent e) {
-		AltosUIPreferences.unregister_ui_listener((AltosUIFrame) e.getWindow());
+		AltosUIFrame frame = (AltosUIFrame) e.getWindow();
+		AltosUIPreferences.unregister_ui_listener(frame);
+		AltosUIFrame.frame_closed();
+		frame.setVisible(false);
+		frame.dispose();
 	}
 }
 
@@ -207,10 +212,79 @@ public class AltosUIFrame extends JFrame implements AltosUIListener, AltosPositi
 			}
 		}
 	}
+
+	int row = 0;
+
+	public void next_row() {
+		row++;
+	}
+
+	int inset = 0;
+
+	public void set_inset(int i) {
+		inset = i;
+	}
+
+	public GridBagConstraints constraints (int x, int width, int fill, int anchor, double weightx, double weighty) {
+		return new GridBagConstraints(x,			/* x */
+					      row,			/* y */
+					      width,			/* width */
+					      1,			/* height */
+					      weightx,			/* weightx */
+					      weighty,			/* weighty */
+					      anchor,			/* anchor */
+					      fill,			/* fill */
+					      new Insets(inset,inset,inset,inset),	/* insets */
+					      0,			/* ipadx */
+					      0);			/* ipady */
+	}
+
+	public GridBagConstraints constraints (int x, int width, int fill, int anchor) {
+		double	weightx = 0;
+		double	weighty = 0;
+
+		if (fill == GridBagConstraints.NONE) {
+			weightx = 0;
+			weighty = 0;
+		} else if (fill == GridBagConstraints.HORIZONTAL) {
+			weightx = 1;
+			weighty = 0;
+		} else if (fill == GridBagConstraints.VERTICAL) {
+			weightx = 0;
+			weighty = 1;
+		} else if (fill == GridBagConstraints.BOTH) {
+			weightx = 1;
+			weighty = 1;
+		}
+
+		return constraints (x, width, fill, anchor, weightx, weighty);
+	}
+
+	public GridBagConstraints constraints (int x, int width, int fill) {
+		return constraints (x, width, fill, GridBagConstraints.WEST);
+	}
+
+	public GridBagConstraints constraints(int x, int width) {
+		return constraints(x, width, GridBagConstraints.NONE);
+	}
+
+	static int open_frames;
+
+	public static void frame_opened() {
+		++open_frames;
+	}
+
+	public static void frame_closed() {
+		--open_frames;
+		if (open_frames == 0)
+			System.exit(0);
+	}
+
 	void init() {
 		AltosUIPreferences.register_ui_listener(this);
 		AltosUIPreferences.register_position_listener(this);
 		position = AltosUIPreferences.position();
+		frame_opened();
 		addWindowListener(new AltosUIFrameListener());
 
 		/* Try to make menus live in the menu bar like regular Mac apps */
