@@ -15,12 +15,12 @@
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
  */
 
-package org.altusmetrum.altosuilib_6;
+package org.altusmetrum.altosuilib_11;
 
 import java.awt.*;
 import libaltosJNI.*;
 
-import org.altusmetrum.altoslib_6.*;
+import org.altusmetrum.altoslib_11.*;
 
 public class AltosUILib extends AltosLib {
 
@@ -112,9 +112,8 @@ public class AltosUILib extends AltosLib {
 
 			String OS = System.getProperty("os.name");
 
-			if (OS.startsWith("Linux")) {
+			if (OS.startsWith("L") || OS.startsWith("W"))
 				has_bluetooth = true;
-			}
 
 			initialized = true;
 		}
