@@ -15,14 +15,14 @@
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
  */
 
-package org.altusmetrum.altosuilib_6;
+package org.altusmetrum.altosuilib_11;
 
 import java.io.File;
 import java.util.prefs.*;
-import org.altusmetrum.altoslib_6.*;
+import org.altusmetrum.altoslib_11.*;
 import javax.swing.filechooser.FileSystemView;
 
-public class AltosUIPreferencesBackend implements AltosPreferencesBackend {
+public class AltosUIPreferencesBackend extends AltosPreferencesBackend {
 
 	private Preferences _preferences = null;
 
@@ -106,4 +106,8 @@ public class AltosUIPreferencesBackend implements AltosPreferencesBackend {
 		/* Use the file system view default directory */
 		return FileSystemView.getFileSystemView().getDefaultDirectory();
 	}
+
+	public void debug(String format, Object ... arguments) {
+		System.out.printf(format, arguments);
+	}
 }
