@@ -20,6 +20,9 @@
 #include <ao_exti.h>
 #include <ao_fec.h>
 #include <ao_packet.h>
+#if HAS_PAD
+#include <ao_pad.h>
+#endif
 
 static uint8_t ao_radio_mutex;
 
@@ -41,7 +44,11 @@ int8_t	ao_radio_rssi;			/* Last received RSSI value */
 
 extern const uint32_t	ao_radio_cal;
 
+#ifdef AO_CC1200_FOSC
+#define FOSC	AO_CC1200_FOSC
+#else
 #define FOSC	40000000
+#endif
 
 #define ao_radio_select()	ao_spi_get_mask(AO_CC1200_SPI_CS_PORT,(1 << AO_CC1200_SPI_CS_PIN),AO_CC1200_SPI_BUS,AO_SPI_SPEED_FAST)
 #define ao_radio_deselect()	ao_spi_put_mask(AO_CC1200_SPI_CS_PORT,(1 << AO_CC1200_SPI_CS_PIN),AO_CC1200_SPI_BUS)
@@ -178,9 +185,9 @@ ao_radio_fifo_write_fixed(uint8_t data, uint8_t len)
 }
 
 static uint8_t
-ao_radio_tx_fifo_space(void)
+ao_radio_int_pin(void)
 {
-	return CC1200_FIFO_SIZE - ao_radio_reg_read(CC1200_NUM_TXBYTES);
+	return ao_gpio_get(AO_CC1200_INT_PORT, AO_CC1200_INT_PIN, AO_CC1200_INT);
 }
 
 static uint8_t
@@ -301,20 +308,28 @@ ao_radio_idle(void)
  *	CHANBW = 5.0 (round to 9.5)
  */
 
+#if FOSC == 40000000
 #define PACKET_SYMBOL_RATE_M		1013008
-
 #define PACKET_SYMBOL_RATE_E_384	8
+#define PACKET_SYMBOL_RATE_E_96		6
+#define PACKET_SYMBOL_RATE_E_24		4
+#endif
+
+#if FOSC == 32000000
+#define PACKET_SYMBOL_RATE_M		239914
+#define PACKET_SYMBOL_RATE_E_384	9
+#define PACKET_SYMBOL_RATE_E_96		7
+#define PACKET_SYMBOL_RATE_E_24		5
+#endif
 
 /* 200 / 2 = 100 */
 #define PACKET_CHAN_BW_384	((CC1200_CHAN_BW_ADC_CIC_DECFACT_12 << CC1200_CHAN_BW_ADC_CIC_DECFACT) | \
 				 (16 << CC1200_CHAN_BW_BB_CIC_DECFACT))
 
-#define PACKET_SYMBOL_RATE_E_96		6
 /* 200 / 10 = 20 */
 #define PACKET_CHAN_BW_96	((CC1200_CHAN_BW_ADC_CIC_DECFACT_48 << CC1200_CHAN_BW_ADC_CIC_DECFACT) | \
 				 (16 << CC1200_CHAN_BW_BB_CIC_DECFACT))
 
-#define PACKET_SYMBOL_RATE_E_24		4
 /* 200 / 25 = 8 */
 #define PACKET_CHAN_BW_24	((CC1200_CHAN_BW_ADC_CIC_DECFACT_48 << CC1200_CHAN_BW_ADC_CIC_DECFACT) | \
 				 (44 << CC1200_CHAN_BW_BB_CIC_DECFACT))
@@ -467,6 +482,7 @@ static const uint16_t rdf_setup[] = {
  */
 #define APRS_SYMBOL_RATE_E	6
 #define APRS_SYMBOL_RATE_M	1013008
+#define APRS_BUFFER_SIZE	64
 
 static const uint16_t aprs_setup[] = {
 	CC1200_DEVIATION_M,	APRS_DEV_M,
@@ -501,6 +517,9 @@ static const uint16_t aprs_setup[] = {
 		 (CC1200_MDMCFG2_SYMBOL_MAP_CFG_MODE_0 << CC1200_MDMCFG2_SYMBOL_MAP_CFG) |
 		 (CC1200_MDMCFG2_UPSAMPLER_P_8 << CC1200_MDMCFG2_UPSAMPLER_P) |
 		 (0 << CC1200_MDMCFG2_CFM_DATA_EN)),
+	CC1200_FIFO_CFG,
+		((0 << CC1200_FIFO_CFG_CRC_AUTOFLUSH) |
+		 (APRS_BUFFER_SIZE << CC1200_FIFO_CFG_FIFO_THR)),
 };
 
 /*
@@ -715,17 +734,11 @@ ao_radio_show_state(char *where)
 static void
 ao_radio_wait_isr(uint16_t timeout)
 {
-	if (timeout)
-		ao_alarm(timeout);
-
 	ao_arch_block_interrupts();
 	while (!ao_radio_wake && !ao_radio_abort)
-		if (ao_sleep(&ao_radio_wake))
+		if (ao_sleep_for(&ao_radio_wake, timeout))
 			ao_radio_abort = 1;
 	ao_arch_release_interrupts();
-
-	if (timeout)
-		ao_clear_alarm();
 }
 
 static void
@@ -806,6 +819,9 @@ ao_radio_test_cmd(void)
 #endif
 #if PACKET_HAS_SLAVE
 		ao_packet_slave_stop();
+#endif
+#if HAS_PAD
+		ao_pad_disable();
 #endif
 		ao_radio_get(0xff);
 		ao_radio_set_mode(AO_RADIO_MODE_TEST);
@@ -831,6 +847,9 @@ ao_radio_test_cmd(void)
 		radio_on = 0;
 #if HAS_MONITOR
 		ao_monitor_enable();
+#endif
+#if HAS_PAD
+		ao_pad_enable();
 #endif
 	}
 }
@@ -846,105 +865,58 @@ ao_radio_send(const void *d, uint8_t size)
 	ao_radio_run();
 }
 
-
-#define AO_RADIO_LOTS	64
-
 void
 ao_radio_send_aprs(ao_radio_fill_func fill)
 {
-	uint8_t	buf[AO_RADIO_LOTS], *b;
+	uint8_t	buf[APRS_BUFFER_SIZE];
 	int	cnt;
 	int	total = 0;
 	uint8_t	done = 0;
 	uint8_t	started = 0;
-	uint8_t	fifo_space;
 
 	ao_radio_abort = 0;
 	ao_radio_get(0xff);
-	fifo_space = CC1200_FIFO_SIZE;
-	while (!done) {
+	ao_radio_wake = 0;
+	while (!done && !ao_radio_abort) {
 		cnt = (*fill)(buf, sizeof(buf));
 		if (cnt < 0) {
 			done = 1;
 			cnt = -cnt;
 		}
-#if CC1200_APRS_TRACE
-		printf("APRS fill %d bytes done %d\n", cnt, done);
-#endif
 		total += cnt;
 
 		/* At the last buffer, set the total length */
 		if (done)
 			ao_radio_set_len(total & 0xff);
 
-		b = buf;
-		while (cnt) {
-			uint8_t	this_len = cnt;
-
-			/* Wait for some space in the fifo */
-			while (!ao_radio_abort && (fifo_space = ao_radio_tx_fifo_space()) == 0) {
-#if CC1200_APRS_TRACE
-				printf("APRS space %d cnt %d\n", fifo_space, cnt); flush();
-#endif
-				ao_radio_wake = 0;
-				ao_radio_wait_isr(AO_MS_TO_TICKS(1000));
-			}
-			if (ao_radio_abort)
-				break;
-			if (this_len > fifo_space)
-				this_len = fifo_space;
-
-			cnt -= this_len;
-
-			if (done) {
-				if (cnt)
-					ao_radio_set_mode(AO_RADIO_MODE_APRS_LAST_BUF);
-				else
-					ao_radio_set_mode(AO_RADIO_MODE_APRS_FINISH);
-			} else
-				ao_radio_set_mode(AO_RADIO_MODE_APRS_BUF);
-
+		/* Wait for some space in the fifo */
+		while (started && ao_radio_int_pin() != 0 && !ao_radio_abort) {
+			ao_radio_wake = 0;
 			ao_exti_enable(AO_CC1200_INT_PORT, AO_CC1200_INT_PIN);
-
-			ao_radio_fifo_write(b, this_len);
-			b += this_len;
-#if CC1200_APRS_TRACE
-			printf("APRS write fifo %d space now %d\n", this_len, ao_radio_tx_fifo_space());
-#endif
-			if (!started) {
-#if CC1200_APRS_TRACE
-				printf("APRS start\n");
-#endif
-				ao_radio_strobe(CC1200_STX);
-#if CC1200_APRS_TRACE
-				{ int t;
-					for (t = 0; t < 20; t++) {
-						uint8_t	status = ao_radio_status();
-						uint8_t space = ao_radio_tx_fifo_space();
-						printf ("status: %02x fifo %d\n", status, space);
-						if ((status >> 4) == 2)
-							break;
-						ao_delay(AO_MS_TO_TICKS(0));
-					}
-				}
-#endif
-				started = 1;
-			}
+			ao_radio_wait_isr(AO_MS_TO_TICKS(1000));
 		}
-		if (ao_radio_abort) {
-			ao_radio_idle();
+		if (ao_radio_abort)
 			break;
+
+		if (done)
+			ao_radio_set_mode(AO_RADIO_MODE_APRS_FINISH);
+		else
+			ao_radio_set_mode(AO_RADIO_MODE_APRS_BUF);
+
+		ao_radio_fifo_write(buf, cnt);
+		if (!started) {
+			ao_radio_strobe(CC1200_STX);
+			started = 1;
 		}
 	}
 	/* Wait for the transmitter to go idle */
-	ao_radio_wake = 0;
-#if CC1200_APRS_TRACE
-	printf("APRS wait idle\n"); flush();
-#endif
-	ao_radio_wait_isr(AO_MS_TO_TICKS(1000));
-#if CC1200_APRS_TRACE
-	printf("APRS abort %d\n", ao_radio_abort);
-#endif
+	while (started && ao_radio_int_pin() != 0 && !ao_radio_abort) {
+		ao_radio_wake = 0;
+		ao_exti_enable(AO_CC1200_INT_PORT, AO_CC1200_INT_PIN);
+		ao_radio_wait_isr(AO_MS_TO_TICKS(1000));
+	}
+	if (ao_radio_abort)
+		ao_radio_idle();
 	ao_radio_put();
 }
 
