@@ -438,6 +438,38 @@
 #define CC1200_IF_MIX_CFG	(CC1200_EXTENDED_BIT | 0x00)
 #define CC1200_FREQOFF_CFG	(CC1200_EXTENDED_BIT | 0x01)
 #define CC1200_TOC_CFG		(CC1200_EXTENDED_BIT | 0x02)
+
+#define CC1200_TOC_CFG_TOC_LIMIT		6
+#define  CC1200_TOC_CFG_TOC_LIMIT_0_2			0
+#define  CC1200_TOC_CFG_TOC_LIMIT_2			1
+#define  CC1200_TOC_CFG_TOC_LIMIT_12			3
+
+#define CC1200_TOC_CFG_TOC_PRE_SYNC_BLOCKLEN	3
+#define  CC1200_TOC_CFG_TOC_PRE_SYNC_BLOCKLEN_8			0
+#define  CC1200_TOC_CFG_TOC_PRE_SYNC_BLOCKLEN_16		1
+#define  CC1200_TOC_CFG_TOC_PRE_SYNC_BLOCKLEN_32		2
+#define  CC1200_TOC_CFG_TOC_PRE_SYNC_BLOCKLEN_64		3
+#define  CC1200_TOC_CFG_TOC_PRE_SYNC_BLOCKLEN_128		4
+#define  CC1200_TOC_CFG_TOC_PRE_SYNC_BLOCKLEN_256		5
+#define  CC1200_TOC_CFG_TOC_PRE_SYNC_BLOCKLEN_8_16		0
+#define  CC1200_TOC_CFG_TOC_PRE_SYNC_BLOCKLEN_6_16		1
+#define  CC1200_TOC_CFG_TOC_PRE_SYNC_BLOCKLEN_2_16		2
+#define  CC1200_TOC_CFG_TOC_PRE_SYNC_BLOCKLEN_1_16		3
+#define  CC1200_TOC_CFG_TOC_PRE_SYNC_BLOCKLEN_1_16_SYNC		4
+
+#define CC1200_TOC_CFG_TOC_POST_SYNC_BLOCKLEN	0
+#define  CC1200_TOC_CFG_TOC_POST_SYNC_BLOCKLEN_8		0
+#define  CC1200_TOC_CFG_TOC_POST_SYNC_BLOCKLEN_16		1
+#define  CC1200_TOC_CFG_TOC_POST_SYNC_BLOCKLEN_32		2
+#define  CC1200_TOC_CFG_TOC_POST_SYNC_BLOCKLEN_64		3
+#define  CC1200_TOC_CFG_TOC_POST_SYNC_BLOCKLEN_128		4
+#define  CC1200_TOC_CFG_TOC_POST_SYNC_BLOCKLEN_256		5
+#define  CC1200_TOC_CFG_TOC_POST_SYNC_BLOCKLEN_FREEZE		0
+#define  CC1200_TOC_CFG_TOC_POST_SYNC_BLOCKLEN_6_32		1
+#define  CC1200_TOC_CFG_TOC_POST_SYNC_BLOCKLEN_2_32		2
+#define  CC1200_TOC_CFG_TOC_POST_SYNC_BLOCKLEN_1_32		3
+#define  CC1200_TOC_CFG_TOC_POST_SYNC_BLOCKLEN_1_32_SYNC	4
+
 #define CC1200_MARC_SPARE	(CC1200_EXTENDED_BIT | 0x03)
 #define CC1200_ECG_CFG		(CC1200_EXTENDED_BIT | 0x04)
 #define CC1200_MDMCFG2		(CC1200_EXTENDED_BIT | 0x05)
