@@ -23,6 +23,10 @@
 #define ao_spi_fast(b)
 #endif
 
+#if !HAS_COMPANION
+#error HAS_COMPANION not set in ao_companion.c
+#endif
+
 #define COMPANION_SELECT()	do {			\
 		ao_spi_get_bit(AO_COMPANION_CS_PORT,	\
 			       AO_COMPANION_CS_PIN,	\
@@ -102,8 +106,7 @@ ao_companion(void)
 		    break;
 	}
 	while (ao_companion_running) {
-		ao_alarm(ao_companion_setup.update_period);
-		if (ao_sleep(DATA_TO_XDATA(&ao_flight_state)))
+		if (ao_sleep_for(DATA_TO_XDATA(&ao_flight_state), ao_companion_setup.update_period))
 			ao_companion_get_data();
 		else
 			ao_companion_notify();
