@@ -78,6 +78,7 @@
 #define HAS_RADIO		0
 #define HAS_TELEMETRY		0
 #define HAS_APRS		0
+#define HAS_COMPANION		1
 
 #define HAS_SPI_1		1
 #define SPI_1_PA5_PA6_PA7	1	/* Barometer */
@@ -320,6 +321,7 @@ struct ao_adc {
  */
 
 #define HAS_MMA655X		1
+#define AO_MMA655X_INVERT	0
 #define AO_MMA655X_SPI_INDEX	AO_SPI_1_PB3_PB4_PB5
 #define AO_MMA655X_CS_PORT	(&stm_gpioc)
 #define AO_MMA655X_CS_PIN	12
