@@ -219,6 +219,16 @@ _ao_config_get(void)
 #if HAS_RADIO_FORWARD
 		if (minor < 21)
 			ao_config.send_frequency = 434550;
+#endif
+#if HAS_APRS
+		if (minor < 22)
+			ao_config.aprs_format = AO_CONFIG_DEFAULT_APRS_FORMAT;
+#endif
+#if HAS_FIXED_PAD_BOX
+		if (minor < 22)
+			ao_config.pad_box = 1;
+		if (minor < 23)
+			ao_config.pad_idle = 120;
 #endif
 		ao_config.minor = AO_CONFIG_MINOR;
 		ao_config_dirty = 1;
@@ -876,8 +886,61 @@ ao_config_aprs_ssid_set(void)
 	ao_config.aprs_ssid = ao_cmd_lex_i;
 	_ao_config_edit_finish();
 }
+
+void
+ao_config_aprs_format_set(void)
+{
+	ao_cmd_decimal();
+	if (ao_cmd_status != ao_cmd_success)
+		return;
+	_ao_config_edit_start();
+	ao_config.aprs_format = ao_cmd_lex_i != 0;
+	_ao_config_edit_finish();
+}
+
+void
+ao_config_aprs_format_show(void)
+{
+	printf ("APRS format: %d\n", ao_config.aprs_format);
+}
 #endif /* HAS_APRS */
 
+#if HAS_FIXED_PAD_BOX
+void
+ao_config_pad_box_show(void)
+{
+	printf ("Pad box: %d\n", ao_config.pad_box);
+}
+
+void
+ao_config_pad_box_set(void)
+{
+	ao_cmd_decimal();
+	if (ao_cmd_status != ao_cmd_success)
+		return;
+	_ao_config_edit_start();
+	ao_config.pad_box = ao_cmd_lex_i;
+	_ao_config_edit_finish();
+}
+
+void
+ao_config_pad_idle_show(void)
+{
+	printf ("Idle timeout: %d\n", ao_config.pad_idle);
+}
+
+void
+ao_config_pad_idle_set(void)
+{
+	ao_cmd_decimal();
+	if (ao_cmd_status != ao_cmd_success)
+		return;
+	_ao_config_edit_start();
+	ao_config.pad_idle = ao_cmd_lex_i;
+	_ao_config_edit_finish();
+}
+#endif
+
 struct ao_config_var {
 	__code char	*str;
 	void		(*set)(void) __reentrant;
@@ -969,6 +1032,14 @@ __code struct ao_config_var ao_config_vars[] = {
 #if HAS_APRS
 	{ "S <ssid>\0Set APRS SSID (0-15)",
 	  ao_config_aprs_ssid_set, ao_config_aprs_ssid_show },
+	{ "C <0 compressed, 1 uncompressed>\0APRS format",
+	  ao_config_aprs_format_set, ao_config_aprs_format_show },
+#endif
+#if HAS_FIXED_PAD_BOX
+	{ "B <box>\0Set pad box (1-99)",
+	  ao_config_pad_box_set, ao_config_pad_box_show },
+	{ "i <seconds>\0Set idle timeout (0 disable)",
+	  ao_config_pad_idle_set, ao_config_pad_idle_show },
 #endif
 	{ "s\0Show",
 	  ao_config_show,		0 },
