@@ -166,6 +166,16 @@ stm_gpio_set(struct stm_gpio *gpio, int pin, uint8_t value) {
 	gpio->bsrr = ((uint32_t) (value ^ 1) << (pin + 16)) | ((uint32_t) value << pin);
 }
 
+static inline void
+stm_gpio_set_bits(struct stm_gpio *gpio, uint16_t bits) {
+	gpio->bsrr = bits;
+}
+
+static inline void
+stm_gpio_clr_bits(struct stm_gpio *gpio, uint16_t bits) {
+	gpio->bsrr = ((uint32_t) bits) << 16;
+}
+
 static inline uint8_t
 stm_gpio_get(struct stm_gpio *gpio, int pin) {
 	return (gpio->idr >> pin) & 1;
@@ -1349,10 +1359,13 @@ extern struct stm_spi stm_spi1, stm_spi2, stm_spi3;
 #define STM_SPI_CR2_TXDMAEN	1
 #define STM_SPI_CR2_RXDMAEN	0
 
+#define STM_SPI_SR_FRE		8
 #define STM_SPI_SR_BSY		7
 #define STM_SPI_SR_OVR		6
 #define STM_SPI_SR_MODF		5
 #define STM_SPI_SR_CRCERR	4
+#define STM_SPI_SR_UDR		3
+#define STM_SPI_SR_CHSIDE	2
 #define STM_SPI_SR_TXE		1
 #define STM_SPI_SR_RXNE		0
 
@@ -1775,7 +1788,7 @@ extern struct stm_tim234 stm_tim2, stm_tim3, stm_tim4;
 #define  STM_TIM234_CCMR1_OC1M_PWM_MODE_1		6
 #define  STM_TIM234_CCMR1_OC1M_PWM_MODE_2		7
 #define  STM_TIM234_CCMR1_OC1M_MASK			7
-#define STM_TIM234_CCMR1_OC1PE	11
+#define STM_TIM234_CCMR1_OC1PE	3
 #define STM_TIM234_CCMR1_OC1FE	2
 #define STM_TIM234_CCMR1_CC1S	0
 #define  STM_TIM234_CCMR1_CC1S_OUTPUT			0
@@ -1815,7 +1828,7 @@ extern struct stm_tim234 stm_tim2, stm_tim3, stm_tim4;
 #define  STM_TIM234_CCMR2_OC3M_PWM_MODE_1		6
 #define  STM_TIM234_CCMR2_OC3M_PWM_MODE_2		7
 #define  STM_TIM234_CCMR2_OC3M_MASK			7
-#define STM_TIM234_CCMR2_OC3PE	11
+#define STM_TIM234_CCMR2_OC3PE	3
 #define STM_TIM234_CCMR2_OC3FE	2
 #define STM_TIM234_CCMR2_CC3S	0
 #define  STM_TIM234_CCMR2_CC3S_OUTPUT			0
