@@ -79,6 +79,7 @@
 #define HAS_RADIO		1
 #define HAS_TELEMETRY		1
 #define HAS_APRS		1
+#define HAS_COMPANION		1
 
 #define HAS_SPI_1		1
 #define SPI_1_PA5_PA6_PA7	1	/* Barometer */
@@ -345,6 +346,7 @@ struct ao_adc {
  */
 
 #define HAS_MMA655X		1
+#define AO_MMA655X_INVERT	0
 #define AO_MMA655X_SPI_INDEX	AO_SPI_1_PE13_PE14_PE15
 #define AO_MMA655X_CS_PORT	(&stm_gpiod)
 #define AO_MMA655X_CS_PIN	4
