@@ -32,6 +32,7 @@
 #if HAS_STACK_GUARD
 #include <ao_mpu.h>
 #endif
+#include <ao_pwm.h>
 
 int
 main(void)
@@ -97,6 +98,8 @@ main(void)
 	ao_sample_profile_init();
 #endif
 	
+	ao_pwm_init();
+
 	ao_start_scheduler();
 	return 0;
 }
