@@ -17,12 +17,13 @@
 
 package org.altusmetrum.telegps;
 
+import java.text.*;
 import java.awt.*;
 import java.awt.event.*;
 import javax.swing.*;
 import javax.swing.event.*;
-import org.altusmetrum.altoslib_6.*;
-import org.altusmetrum.altosuilib_6.*;
+import org.altusmetrum.altoslib_11.*;
+import org.altusmetrum.altosuilib_11.*;
 
 public class TeleGPSConfigUI
 	extends AltosUIDialog
@@ -40,6 +41,7 @@ public class TeleGPSConfigUI
 	JLabel			rate_label;
 	JLabel			aprs_interval_label;
 	JLabel			aprs_ssid_label;
+	JLabel			aprs_format_label;
 	JLabel			flight_log_max_label;
 	JLabel			callsign_label;
 	JLabel			tracker_motion_label;
@@ -52,11 +54,12 @@ public class TeleGPSConfigUI
 	JLabel			version_value;
 	JLabel			serial_value;
 	AltosUIFreqList		radio_frequency_value;
-	JTextField		radio_calibration_value;
+	JLabel			radio_calibration_value;
 	JRadioButton		radio_enable_value;
 	AltosUIRateList		rate_value;
 	JComboBox<String>	aprs_interval_value;
 	JComboBox<Integer>	aprs_ssid_value;
+	JComboBox<String>	aprs_format_value;
 	JComboBox<String>	flight_log_max_value;
 	JTextField		callsign_value;
 	JComboBox<String>	tracker_motion_value;
@@ -135,13 +138,6 @@ public class TeleGPSConfigUI
 		return product != null && product.startsWith("TeleGPS");
 	}
 
-	void set_radio_calibration_tool_tip() {
-		if (radio_calibration_value.isEnabled())
-			radio_calibration_value.setToolTipText("Tune radio output to match desired frequency");
-		else
-			radio_calibration_value.setToolTipText("Cannot tune radio while connected over packet mode");
-	}
-
 	void set_radio_enable_tool_tip() {
 		if (radio_enable_value.isEnabled())
 			radio_enable_value.setToolTipText("Enable/Disable telemetry and RDF transmissions");
@@ -165,11 +161,20 @@ public class TeleGPSConfigUI
 
 	void set_aprs_ssid_tool_tip() {
 		if (aprs_ssid_value.isEnabled())
-			aprs_interval_value.setToolTipText("Set the APRS SSID (secondary station identifier)");
-		else if (aprs_interval_value.isEnabled())
-			aprs_interval_value.setToolTipText("Software version doesn't support setting the APRS SSID");
+			aprs_ssid_value.setToolTipText("Set the APRS SSID (secondary station identifier)");
+		else if (aprs_ssid_value.isEnabled())
+			aprs_ssid_value.setToolTipText("Software version doesn't support setting the APRS SSID");
 		else
-			aprs_interval_value.setToolTipText("Hardware doesn't support APRS");
+			aprs_ssid_value.setToolTipText("Hardware doesn't support APRS");
+	}
+
+	void set_aprs_format_tool_tip() {
+		if (aprs_format_value.isEnabled())
+			aprs_format_value.setToolTipText("Set the APRS format (compressed/uncompressed)");
+		else if (aprs_format_value.isEnabled())
+			aprs_format_value.setToolTipText("Software version doesn't support setting the APRS format");
+		else
+			aprs_format_value.setToolTipText("Hardware doesn't support APRS");
 	}
 
 	void set_flight_log_max_tool_tip() {
@@ -304,10 +309,8 @@ public class TeleGPSConfigUI
 		c.anchor = GridBagConstraints.LINE_START;
 		c.insets = ir;
 		c.ipady = 5;
-		radio_calibration_value = new JTextField(String.format("%d", 1186611));
-		radio_calibration_value.getDocument().addDocumentListener(this);
+		radio_calibration_value = new JLabel(String.format("%d", 1186611));
 		pane.add(radio_calibration_value, c);
-		set_radio_calibration_tool_tip();
 		row++;
 
 		/* Radio Enable */
@@ -413,6 +416,33 @@ public class TeleGPSConfigUI
 		set_aprs_ssid_tool_tip();
 		row++;
 
+		/* APRS format */
+		c = new GridBagConstraints();
+		c.gridx = 0; c.gridy = row;
+		c.gridwidth = 4;
+		c.fill = GridBagConstraints.NONE;
+		c.anchor = GridBagConstraints.LINE_START;
+		c.insets = il;
+		c.ipady = 5;
+		aprs_format_label = new JLabel("APRS format:");
+		pane.add(aprs_format_label, c);
+
+		c = new GridBagConstraints();
+		c.gridx = 4; c.gridy = row;
+		c.gridwidth = 4;
+		c.fill = GridBagConstraints.HORIZONTAL;
+		c.weightx = 1;
+		c.anchor = GridBagConstraints.LINE_START;
+		c.insets = ir;
+		c.ipady = 5;
+		aprs_format_value = new JComboBox<String>(AltosLib.ao_aprs_format_name);
+		aprs_format_value.setEditable(false);
+		aprs_format_value.addItemListener(this);
+		aprs_format_value.setMaximumRowCount(AltosLib.ao_aprs_format_name.length);
+		pane.add(aprs_format_value, c);
+		set_aprs_format_tool_tip();
+		row++;
+
 		/* Callsign */
 		c = new GridBagConstraints();
 		c.gridx = 0; c.gridy = row;
@@ -648,7 +678,11 @@ public class TeleGPSConfigUI
 			String motion = tracker_motion_value.getSelectedItem().toString();
 			tracker_motion_label.setText(get_tracker_motion_label());
 			set_tracker_motion_values();
-			set_tracker_motion((int) (AltosConvert.height.parse(motion, !imperial_units) + 0.5));
+			try {
+				int m = (int) (AltosConvert.height.parse_locale(motion, !imperial_units) + 0.5);
+				set_tracker_motion(m);
+			} catch (ParseException pe) {
+			}
 		}
 		if (!was_dirty)
 			set_clean();
@@ -717,10 +751,6 @@ public class TeleGPSConfigUI
 		}
 	}
 
-	public int radio_calibration() throws AltosConfigDataException {
-		return parse_int("radio calibration", radio_calibration_value.getText(), false);
-	}
-
 	public void set_radio_enable(int new_radio_enable) {
 		if (new_radio_enable >= 0) {
 			radio_enable_value.setSelected(new_radio_enable > 0);
@@ -824,7 +854,7 @@ public class TeleGPSConfigUI
 	}
 
 	String get_tracker_motion_label() {
-		return String.format("Logging Trigger Motion (%s):", AltosConvert.height.show_units());
+		return String.format("Logging Trigger Motion (%s):", AltosConvert.height.parse_units());
 	}
 
 	void set_tracker_tool_tip() {
@@ -848,7 +878,12 @@ public class TeleGPSConfigUI
 	}
 
 	public int tracker_motion() throws AltosConfigDataException {
-		return (int) AltosConvert.height.parse(tracker_motion_value.getSelectedItem().toString());
+		String str = tracker_motion_value.getSelectedItem().toString();
+		try {
+			return (int) (AltosConvert.height.parse_locale(str) + 0.5);
+		} catch (ParseException pe) {
+			throw new AltosConfigDataException("invalid tracker motion %s", str);
+		}
 	}
 
 	public void set_tracker_interval(int tracker_interval) {
@@ -894,4 +929,16 @@ public class TeleGPSConfigUI
 		Integer i = (Integer) aprs_ssid_value.getSelectedItem();
 		return i;
 	}
+
+	public void set_aprs_format(int new_aprs_format) {
+		aprs_format_value.setVisible(new_aprs_format >= 0);
+		aprs_format_label.setVisible(new_aprs_format >= 0);
+
+		aprs_format_value.setSelectedIndex(Math.max(0,new_aprs_format));
+		set_aprs_format_tool_tip();
+	}
+
+	public int aprs_format() throws AltosConfigDataException {
+		return aprs_format_value.getSelectedIndex();
+	}
 }
