@@ -21,8 +21,8 @@ import java.util.*;
 import java.awt.*;
 import java.awt.event.*;
 import javax.swing.*;
-import org.altusmetrum.altoslib_6.*;
-import org.altusmetrum.altosuilib_6.*;
+import org.altusmetrum.altoslib_11.*;
+import org.altusmetrum.altosuilib_11.*;
 
 public class TeleGPSState extends AltosUIFlightTab {
 
@@ -136,10 +136,13 @@ public class TeleGPSState extends AltosUIFlightTab {
 
 	class FlightLogMax extends AltosUIIndicator {
 		public void show(AltosState state, AltosListenerState listener_state) {
-			if (state.flight_log_max == AltosLib.MISSING)
+			int storage = state.flight_log_max;
+			if (storage == AltosLib.MISSING)
+				storage = state.log_space >> 10;
+			if (storage == AltosLib.MISSING)
 				show("Missing");
 			else
-				show(String.format("%dkB", state.flight_log_max));
+				show(String.format("%dkB", storage));
 		}
 
 		public FlightLogMax(Container container, int y) {
@@ -161,6 +164,26 @@ public class TeleGPSState extends AltosUIFlightTab {
 		}
 	}
 
+	class ReceiverBattery extends AltosUIVoltageIndicator {
+
+		public double voltage(AltosState state) { return AltosLib.MISSING; }
+
+		public double good() { return AltosLib.ao_battery_good; }
+
+		public boolean hide(AltosState state, AltosListenerState listener_state, int i) {
+			return value(state, listener_state, i) == AltosLib.MISSING;
+		}
+
+		public double value(AltosState state, AltosListenerState listener_state, int i) {
+			if (listener_state == null)
+				return AltosLib.MISSING;
+			return listener_state.battery;
+		}
+
+		public ReceiverBattery (AltosUIFlightTab container, int y) {
+			super(container, y, "Receiver Battery", 2);
+		}
+	}
 
 	public void labels(Container container, int y) {
 		GridBagLayout		layout = (GridBagLayout)(container.getLayout());
@@ -203,5 +226,6 @@ public class TeleGPSState extends AltosUIFlightTab {
 		add(new FirmwareVersion(this, y++));
 		add(new FlightLogMax(this, y++));
 		add(new BatteryVoltage(this, y++));
+		add(new ReceiverBattery(this, y++));
 	}
 }
